//
//  KST1012.h
//  KST1012 - Proximity Tag SDK
//
//  Copyright (c) 2013 KS Technologies, LLC. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreBluetooth/CoreBluetooth.h>

#import "KSTBLEManager.h"

/* TO DO
 1.  Add in receivedAlert Capability
 2.  Add in a timer to update signal strength dynamically
 */

// Delegate Methods
@protocol KST1012Delegate <NSObject>
-(void)receivedAlert;
-(void)updatedBatteryLevelTo:(KSTBLEBatteryLevel)newBatteryLevel;
@end

@interface KST1012 : NSObject <CBPeripheralDelegate> {
    
}

// Delegate
@property( nonatomic, strong ) id <KST1012Delegate> delegate;

// Properties
@property( nonatomic, readonly ) KSTBLEDeviceTypes *deviceType;             // KST1012_ProximityTag
@property( nonatomic ) CBPeripheral *peripheral;
@property( nonatomic, readonly ) NSMutableDictionary *deviceInformation;
@property( nonatomic, readonly ) NSNumber *isProximityAlertEnabled;         // boolValue
@property( nonatomic, readonly ) NSNumber *isLinkLossAlertEnabled;          // boolValue
@property( nonatomic, readonly ) NSNumber *isConnected;                     // boolValue
@property( nonatomic, readonly ) KSTBLESignalStrength *signalStrength;
@property( nonatomic, readonly ) KSTBLEBatteryLevel *batteryLevel;

// Methods
-(void)establishConnection;
-(void)soundProximityAlert;
-(void)silenceProximityAlert;
-(void)enableLinkLossAlert;
-(void)disableLinkLossAlert;

@end
