//
//  KST1012.m
//  KST1012 - Proximity Tag SDK
//
//  Copyright (c) 2013 KS Technologies, LLC. All rights reserved.
//

#import "KST1012.h"
#import <CoreBluetooth/CoreBluetooth.h>

@implementation KST1012

#pragma mark - Initialization Methods
- (id)init
{
    self = [super init];
    
    if (self)
    {
        
    }
    
    return self;
}

-(void)establishConnection {
    [_peripheral setDelegate:self];
    [_peripheral discoverServices:nil];
}

#pragma mark - Methods
-(void)soundProximityAlert {
    
    char byteDataTest[1];
    byteDataTest[0] = 2;
    
    NSData *sendData = [NSData dataWithBytes:byteDataTest length:1];
    
    if( _peripheral.isConnected && [_peripheral.services count]>0 ) {
        for (CBService *aService in _peripheral.services) {
            if ([aService.UUID isEqual:[CBUUID UUIDWithString:@"1802"]]) {
                if( [aService.characteristics count] > 0 ) {
                    for( CBCharacteristic *aCharacteristic in aService.characteristics ) {                        
                        if ([aCharacteristic.UUID isEqual:[CBUUID UUIDWithString:@"2A06"]]) {
                            [_peripheral writeValue:sendData forCharacteristic:aCharacteristic type:CBCharacteristicWriteWithoutResponse ];
                        }
                    }
                }
            }
        }
        
    }
}

-(void)silenceProximityAlert {
    char byteDataTest[1];
    byteDataTest[0] = 0;
    
    NSData *sendData = [NSData dataWithBytes:byteDataTest length:1];
    
    if( _peripheral.isConnected && [_peripheral.services count]>0 ) {
        for (CBService *aService in _peripheral.services) {
            if ([aService.UUID isEqual:[CBUUID UUIDWithString:@"1802"]]) {
                if( [aService.characteristics count] > 0 ) {
                    for( CBCharacteristic *aCharacteristic in aService.characteristics ) {
                        if ([aCharacteristic.UUID isEqual:[CBUUID UUIDWithString:@"2A06"]]) {
                            [_peripheral writeValue:sendData forCharacteristic:aCharacteristic type:CBCharacteristicWriteWithoutResponse ];
                        }
                    }
                }
            }
        }
        
    }
}

-(void)enableLinkLossAlert {
    
    char byteDataTest[1];
    byteDataTest[0] = 2;
    
    NSData *sendData = [NSData dataWithBytes:byteDataTest length:1];
    
    if( _peripheral.isConnected && [_peripheral.services count]>0 ) {
        for (CBService *aService in _peripheral.services) {
            if ([aService.UUID isEqual:[CBUUID UUIDWithString:@"1803"]]) {
                if( [aService.characteristics count] > 0 ) {
                    for( CBCharacteristic *aCharacteristic in aService.characteristics ) {
                        if ([aCharacteristic.UUID isEqual:[CBUUID UUIDWithString:@"2A06"]]) {
                            [_peripheral writeValue:sendData forCharacteristic:aCharacteristic type:CBCharacteristicWriteWithoutResponse ];
                        }
                    }
                }
            }
        }
        
    }
}

-(void)disableLinkLossAlert {
    char byteDataTest[1];
    byteDataTest[0] = 0;
    
    NSData *sendData = [NSData dataWithBytes:byteDataTest length:1];
    
    if( _peripheral.isConnected && [_peripheral.services count]>0 ) {
        for (CBService *aService in _peripheral.services) {
            if ([aService.UUID isEqual:[CBUUID UUIDWithString:@"1803"]]) {
                if( [aService.characteristics count] > 0 ) {
                    for( CBCharacteristic *aCharacteristic in aService.characteristics ) {
                        if ([aCharacteristic.UUID isEqual:[CBUUID UUIDWithString:@"2A06"]]) {
                            [_peripheral writeValue:sendData forCharacteristic:aCharacteristic type:CBCharacteristicWriteWithoutResponse ];
                        }
                    }
                }
            }
        }
        
    }
}

#pragma mark - CBPeripheralDelegate Methods
- (void)peripheral:(CBPeripheral *)aPeripheral didDiscoverServices:(NSError *)error {
    
    for (CBService *aService in aPeripheral.services) {
        [aPeripheral discoverCharacteristics:nil forService:aService];
    }
    
}

- (void) peripheral:(CBPeripheral *)aPeripheral didDiscoverCharacteristicsForService:(CBService *)service error:(NSError *)error {
    
    for (CBCharacteristic *aChar in service.characteristics) {
        [aPeripheral readValueForCharacteristic:aChar];
        [aPeripheral setNotifyValue:YES forCharacteristic:aChar];
    }
    
    // You can now interact with the Proximity Device.
    
}

- (void) peripheral:(CBPeripheral *)aPeripheral didUpdateValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error {
    
    if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:@"2A06"]]) {
        // Alert Level
        if( [characteristic.value length] > 0 ) {
            NSLog(@"%@ is ready for action", aPeripheral.name);
        }
        
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:@"2A19"]]) {
        
        // Battery Level
        NSLog(@"BAT %@", characteristic.value);
        if( [characteristic.value length] > 0 ) {
            
            int batteryInt = *(int*)([characteristic.value bytes]);
            
            if( _delegate && [_delegate respondsToSelector:@selector(updatedBatteryLevelTo:)] ) {
                
                if( batteryInt > 0 && batteryInt <= 25 ) {
                    [_delegate updatedBatteryLevelTo:KSTBLEBatteryLevel0_25];
                    
                } else if( batteryInt > 25 && batteryInt <= 50 ) {
                    [_delegate updatedBatteryLevelTo:KSTBLEBatteryLevel25_50];
                    
                } else if( batteryInt > 50 && batteryInt <= 75 ) {
                    [_delegate updatedBatteryLevelTo:KSTBLEBatteryLevel50_75];
                    
                } else if( batteryInt > 75 && batteryInt <= 100 ) {
                    [_delegate updatedBatteryLevelTo:KSTBLEBatteryLevel75_100];
                }
            }

        }
        
    } else if ([characteristic.UUID isEqual:[CBUUID UUIDWithString:@"2A07"]]) {
        // Tx Power
        if( [characteristic.value length] > 0 ) {
        }
    }
    
}

- (void)peripheral:(CBPeripheral *)aPeripheral didWriteValueForCharacteristic:(CBCharacteristic *)characteristic error:(NSError *)error {
    NSLog(@"proxTAG:  Wrote Value %@ for Characteristic %@ on %@ (%@) With Error %@", characteristic.value, characteristic.UUID, aPeripheral.name, aPeripheral.UUID, [error description]);
}

@end
