//
//  KSTBLEManager.h
//
//  Copyright (c) 2013 KS Technologies, LLC. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <CoreBluetooth/CoreBluetooth.h>

/* TO DO
 1.  Add in KSTBLEDeviceTypes support
 2.  Add in KSTBLEManagerState support
 3.  Add in proximity tag alert capability
 4.  Add in CBCentralManager error handling (didFailToConnect, etc)
 */

// Notififcations; payload includes the specific object that was connected/disconnected.
#define kKSTBLEManagerDiscoveredNewDevice       @"kKSTBLEManagerDiscoveredNewDevice"
#define kKSTBLEMAnagerConnectedToDevice         @"kKSTBLEMAnagerConnectedToDevice"
#define kKSTBLEManagerDisconnectedFromDevice    @"kKSTBLEManagerDisconnectedFromDevice"

typedef enum {
    KST1010_Gumstick,           // BLE Gumstick
    KST1011_ArduinoShield,      // BLE Arduino Shield
    KST1012_ProximityTag,       // BLE Proximity Tag
    KST1013_ThermometerBeacon,  // BLE Temperature Sensor
    KST1014_Secret14,           // BLE -= In Development =-
    KST1015_Secret15            // BLE -= In Development =-
} KSTBLEDeviceTypes;

typedef enum {
    KSTBLEManagerStateOff,
    KSTBLEManagerStateScanning,
    KSTBLEManagerStateOn
} KSTBLEManagerState;

typedef enum {
    KSTBLESignalStrength0_25,
    KSTBLESignalStrength25_50,
    KSTBLESignalStrength50_75,
    KSTBLESignalStrength75_100
} KSTBLESignalStrength;

typedef enum {
    KSTBLEBatteryLevel0_25,
    KSTBLEBatteryLevel25_50,
    KSTBLEBatteryLevel50_75,
    KSTBLEBatteryLevel75_100
} KSTBLEBatteryLevel;

@interface KSTBLEManager : NSObject <CBCentralManagerDelegate> {
    
    NSMutableArray *_devicesToFind;
    
    @private
    CBCentralManager *_bleManager;
    
}

// Singleton
+ (id)manager;

// Properties
@property( nonatomic ) NSMutableArray *visibleDevices;
@property( nonatomic ) NSMutableArray *devicesToFind;
@property( nonatomic ) NSNumber *isBLEOperational;
@property( nonatomic ) KSTBLEManagerState *managerState;

// Methods
-(void)KSTBLEManagerInitialize;
-(void)KSTBLEManagerStartScanForDevices;
-(void)KSTBLEManagerStopScanForDevices;
-(void)KSTBLEManagerConnectToDevice:(id)aKSTDevice;
-(void)KSTBLEManagerDisconnectedFromDevice:(id)aKSTDevice;

// Private Objects
@property( nonatomic ) CBCentralManager *bleManager;

@end
