//
//  KSTBLEManager.m
//
//  Copyright (c) 2013 KS Technologies. All rights reserved.
//

#import "KSTBLEManager.h"
#import <CoreBluetooth/CoreBluetooth.h>

@implementation KSTBLEManager

static KSTBLEManager *manager = nil;

#pragma mark - Singleton Creation
+ (KSTBLEManager *)manager {
    
    if (nil != manager) {
        return manager;
    }
    
    static dispatch_once_t pred;
    dispatch_once(&pred, ^{
        manager = [[KSTBLEManager alloc] init];
    });
    
    return manager;
}

#pragma mark - Methods
-(void)KSTBLEManagerInitialize {
    if( !_bleManager ) {
        _bleManager = [[CBCentralManager alloc] initWithDelegate:self queue:dispatch_get_main_queue()];
    }
    
    if( !_visibleDevices ) {
        _visibleDevices = [NSMutableArray array];
    }
    
    if( !_devicesToFind ) {
        _devicesToFind = [NSMutableArray array];
    }
    
    if( !_isBLEOperational ) {
        _isBLEOperational = [NSNumber numberWithBool:NO];
    }
    
    if( !_managerState ) {
        _managerState = KSTBLEManagerStateOff;
    }
}

-(void)KSTBLEManagerStartScanForDevices {
    
    // Start looking for cool stuff ... if you have the radio for it.
    if( [self isLECapableHardware] ) {
        _isBLEOperational = [NSNumber numberWithBool:YES];
        
        if( [_devicesToFind containsObject:[NSNumber numberWithInt:KST1012_ProximityTag] ] )
        {
            // We are looking here for the Immediate Alert Service.  Checkout developer.bluetooth.org for all possible BLE services.
            [_bleManager scanForPeripheralsWithServices:[NSArray arrayWithObjects:[CBUUID UUIDWithString:@"1802"], nil]
                                                options:[NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:CBCentralManagerScanOptionAllowDuplicatesKey] ];
        }
        // Clearly more work to do here; we need to add in more of the KST Products here, but all in good time!
        
    }
    
}

-(void)KSTBLEManagerConnectToDevice:(id)aKSTDevice {
    [_bleManager retrieveConnectedPeripherals];
    [_bleManager retrievePeripherals:[NSArray arrayWithObject:aKSTDevice]];
    [_bleManager connectPeripheral:aKSTDevice options:[NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:CBConnectPeripheralOptionNotifyOnDisconnectionKey]];
}

-(void)KSTBLEManagerDisconnectedFromDevice:(id)aKSTDevice {
    [_bleManager cancelPeripheralConnection:aKSTDevice];
}

#pragma mark - Convenience Methods
- (BOOL)isLECapableHardware
{
    bool isBLEready = FALSE;
    
    NSString *state = nil;
    
    switch ([_bleManager state])
    {
        case CBCentralManagerStateUnsupported:
            state = @"The platform/hardware doesn't support Bluetooth Low Energy.";
            break;
        case CBCentralManagerStateUnauthorized:
            state = @"The app is not authorized to use Bluetooth Low Energy.";
            break;
        case CBCentralManagerStatePoweredOff:
            state = @"Bluetooth is currently powered off.";
            break;
        case CBCentralManagerStatePoweredOn:
        {
            state = @"Bluetooth LE is ready for action.";
            isBLEready = TRUE;
        } break;
        case CBCentralManagerStateUnknown:
            state = @"Bluetooth LE state is unknown.";
            break;
        default:
            state = @"Bluetooth LE state is unknown.";
            break;
            
    }

    if( isBLEready ) {
        _isBLEOperational = [NSNumber numberWithBool:YES];
    } else {
        _isBLEOperational = [NSNumber numberWithBool:NO];
    }
    
    NSLog(@"%@", state);
    
    return isBLEready;
}

#pragma mark - CBCentralManager Delegate Methods
- (void)centralManagerDidUpdateState:(CBCentralManager *)central
{
    if( [self isLECapableHardware] ) {
        _isBLEOperational = [NSNumber numberWithBool:YES];

        // Clearly more work to do here; we need to add in more of the KST Products here, but all in good time!
        [_bleManager scanForPeripheralsWithServices:nil
                                            options:[NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:CBCentralManagerScanOptionAllowDuplicatesKey] ];
        
    } else {
        _isBLEOperational = [NSNumber numberWithBool:NO];
    }
}

- (void)centralManager:(CBCentralManager *)central didDiscoverPeripheral:(CBPeripheral *)aPeripheral advertisementData:(NSDictionary *)advertisementData RSSI:(NSNumber *)RSSI
{
    
    NSLog(@"Advertisement for %@ (%@ dBm)", aPeripheral.name , RSSI);
    
	if ( ![_visibleDevices containsObject:aPeripheral] && !aPeripheral.isConnected ) {
        [_visibleDevices addObject:aPeripheral];
        
        // The array of visible devices has been updated; let everyone know.
        NSDictionary *newDevicePayload = [NSDictionary dictionaryWithObject:aPeripheral forKey:@"peripheral"];
        [[NSNotificationCenter defaultCenter] postNotificationName:kKSTBLEManagerDiscoveredNewDevice object:nil userInfo:newDevicePayload];
        
	}
    
}

- (void)centralManager:(CBCentralManager *)central didRetrievePeripherals:(NSArray *)peripherals {
        
    for (CBPeripheral *peripheral in peripherals) {
        [_bleManager connectPeripheral:peripheral options:[NSDictionary dictionaryWithObject:[NSNumber numberWithBool:YES] forKey:CBConnectPeripheralOptionNotifyOnDisconnectionKey]];
    }
    
}

- (void)centralManager:(CBCentralManager *)central didConnectPeripheral:(CBPeripheral *)aPeripheral {
    
    NSLog(@"Connected to %@", [aPeripheral  name]);

    NSDictionary *newDevicePayload = [NSDictionary dictionaryWithObject:aPeripheral forKey:@"peripheral"];
    [[NSNotificationCenter defaultCenter] postNotificationName:kKSTBLEMAnagerConnectedToDevice object:nil userInfo:newDevicePayload];
    
}

@end
